@@ -1,14 +1,25 @@
 from __future__ import unicode_literals
+"""
+Main function of this project.
+"""
 import os
 import sys
 import eel
 from pathlib import Path
-from tkinter import Tk, filedialog
 import youtube_dl as yt
 import configparser
 from threading import Thread
 import datetime
-from . import __version__ as version
+import urllib.request
+import json
+from distutils.version import LooseVersion
+from . import __version__ as VERSION
+try:    # check if tkinter is present. if not print error
+    from tkinter import Tk, filedialog
+except ImportError:
+    print('Error: tkinter not found')
+    print('On linux you can install tkinter by executing:')
+    print('sudo apt-get install python3-tk')
 
 
 class MyLogger(object):
@@ -43,13 +54,13 @@ config_path = os.path.dirname(os.path.realpath(__file__)) + '/' + config_locatio
 def download(url):
     # YouTube_dl options
     ydl_opts = {
-                'verbose': 'true',
-                'noplaylist': 'true',
-                'format': 'best',
-                'outtmpl': get_save_path() + '/%(title)s.%(ext)s',
-                'progress_hooks': [hook],
-                'logger': MyLogger(),
-                }
+        'verbose': 'true',
+        'noplaylist': 'true',
+        'format': 'best',
+        'outtmpl': get_save_path() + '/%(title)s.%(ext)s',
+        'progress_hooks': [hook],
+        'logger': MyLogger(),
+        }
 
     try:
         with yt.YoutubeDL(ydl_opts) as ydl:
@@ -76,11 +87,18 @@ def hook(d):
         eel.update_status('')
 
         if d['status'] == 'downloading':
-            filename = os.path.basename(d['filename'])
-            speed = str(round(d['speed'] / 1000000, 2))  # speed in mb/s
-            elapsed_time = str(datetime.timedelta(seconds=round(d['elapsed']))) # elapsed time
-            estimated_time = str(datetime.timedelta(seconds=d['eta'])) # estimated time
-            eel.update_status('Downloading ...\nSpeed: {} mb/s | {} / {}'.format(speed, elapsed_time, estimated_time))
+            try:
+                filename = os.path.basename(d['filename'])
+                speed = round(d['speed'] / 1000000, 2)  # speed in mb/s
+                elapsed_time = datetime.timedelta(seconds=round(d['elapsed'])) # elapsed time
+                estimated_time = datetime.timedelta(seconds=d['eta']) # estimated time
+            except TypeError:
+                speed = 0   # backup values because the above ones sometimes fail
+                elapsed_time = 0
+                estimated_time = 0
+
+            eel.update_status('Downloading ...\nSpeed: {:.2f} mb/s | {} / {}'. \
+                              format(speed, elapsed_time, estimated_time))
 
         if d['status'] == 'finished':
             eel.update_status('Download completed successfully ✔️')
@@ -122,7 +140,7 @@ def update_status_output():
 # updates version badge with current version (gets called in html body onload)
 @eel.expose
 def update_version_badge():
-    eel.update_version_badge('v' + version)
+    eel.update_version_badge('v' + VERSION)
 
 
 # checks if config file exists. if not creates it
@@ -132,13 +150,35 @@ def check_config():
         with open(config_path, 'a') as f:
             f.write('[MAIN]\nsave_path = \n')
 
+def check_for_update():
+    current_version = VERSION
+    latest_version = ''
+    url = 'https://pypi.org/pypi/sytd/json' # pypi json url
+    try:
+        with urllib.request.urlopen(url) as request:
+            latest_version = json.loads(request.read().decode())['info']['version']
+    except:
+        pass
+
+    if LooseVersion(current_version) < LooseVersion(latest_version):
+        print('Version {} of sytd is available!'.format(latest_version))
+        eel.show_update_available()
+
+def close(path,sockets):
+    print('Bye...')
+    sys.exit(0)
 
 def run():
     check_config()
+    check_for_update()
     try:
-        eel.start('main.html', mode='chrome', port=0, size=(600, 820))
+        eel.start('main.html', mode='chrome', port=0, size=(600, 840),
+                  close_callback=close)
     except (SystemExit, KeyboardInterrupt):
         pass
+    except OSError:
+        print('Chrome is required to run this program. Install chrome here: '
+              'https://www.google.com/chrome/')
 
 
 if __name__ == '__main__':
